#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>
#include <sysexits.h>
#include <getopt.h>
#include <arpa/inet.h>
#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include "declrom.h"

void usage(const char *pname) {
	fprintf(stderr, "%s: [-i|--input] <filename>\n", pname);
	return;
}

int main(int argc, char *argv[]) {
	char c = 0;
	int loptind = 0;
	char *infile = NULL;
	struct option o[] = {
		{"help", 0, 0, 'h'},
		{"input", 1, 0, 'i'},
		{0,0,0,0}
	};

	while( (c = getopt_long(argc, argv, "hi:", o, &loptind)) != -1 ) {
		switch(c) {
		case 'i':
			if(!optarg) {
				usage(argv[0]);
				exit(EX_USAGE);
			}
			infile = optarg;
			break;
		case 'h':
			usage(argv[0]);
			exit(EX_USAGE);
			break;
		}
	}

	if(!infile) {
		usage(argv[0]);
		exit(EX_USAGE);
	}
	errno = 0;

	sDeclRom *rom = sdeclrom_readfile(infile);
	if(!rom) {
		fprintf(stderr, "Error reading rom file\n");
		exit(EX_CONFIG);
	}

	printf("Format block:\n");
	printf("bytelanes = 0x%x\n", rom->formatblock.bytelanes);
	printf("testpattern = 0x%x\n", rom->formatblock.testpattern);
	printf("format = 0x%x\n", rom->formatblock.format);
	printf("revision = 0x%x\n", rom->formatblock.revision);
	printf("crc = 0x%x\n", rom->formatblock.crc);
	printf("length = %d\n", rom->formatblock.length);
	printf("offset = %d\n", rom->formatblock.diroffset);

	sResource *iter;
	sResource *boardrsrc = NULL;
	TAILQ_FOREACH(iter, &rom->resourcelist, next) {
		if(!iter) break;
		if(iter->rsrcid == 255) break;

		printf("Resource %d\n", iter->rsrcid);
		
		if(iter->type) {
			printf("\tsRsrcType: %d %d %d %d\n", iter->type->category, iter->type->cType, iter->type->drSW, iter->type->drHW);
		}
		if(iter->name) {
			printf("\tsRsrcName: %s\n", iter->name);
		}
		if(iter->drvrDir[0]) {
			int curdrv;
			printf("\tRsrcDrvrDir:\n");
			for(curdrv = 0; iter->drvrDir[curdrv]; curdrv++) {
				printf("\t\tid: %d, len: %d\n", iter->drvrDir[curdrv]->cpuid, iter->drvrDir[curdrv]->len);
			}
		}
		if(iter->bootRec) {
			printf("\tsRsrcBootRec: cpuid: %d len: %d\n", iter->bootRec->cpuid, iter->bootRec->len);
		}
		if(iter->flags) {
			printf("\tsRsrcFlags: 0x%x\n", iter->flags & 0x00FFFFFF);
		}
		if(iter->devid) {
			printf("\tsRsrcHWDevId: %d\n", iter->devid & 0x00FFFFFF);
		}
		if(iter->minBaseOS) {
			printf("\tMinorBaseOS: 0x%x\n", *iter->minBaseOS);
		}
		if(iter->minLen) {
			printf("\tMinorLength: 0x%x\n", *iter->minLen);
		}
		if(iter->boardid) {
			printf("\tBoardId: %d\n", iter->boardid);
		}
		if(iter->primaryInit) {
			printf("\tPrimaryInit: cpuid: %d len: %d\n", iter->primaryInit->cpuid, iter->primaryInit->len);
		}
		if(iter->secondaryInit) {
			printf("\tSecondaryInit: cpuid: %d len: %d\n", iter->secondaryInit->cpuid, iter->secondaryInit->len);
		}
		if(iter->vendorid || iter->serialnum || iter->revlevel || iter->partnum || iter->date) {
			printf("\tVendorInfo:\n");
		}
		if(iter->vendorid) {
			printf("\t\tVendorID: %s\n", iter->vendorid);
		}
		if(iter->serialnum) {
			printf("\t\tSerialNum: %s\n", iter->serialnum);
		}
		if(iter->revlevel) {
			printf("\t\tPartNum: %s\n", iter->partnum);
		}
		if(iter->date) {
			printf("\t\tDate: %s\n", iter->date);
		}

		if(iter->hasVidNames) {
			sRsrcVidNameEntry *videntry;
			boardrsrc = iter;
			printf("\t\tVideo Mode entry & Name:\n");
			TAILQ_FOREACH(videntry, &iter->vidNames, next) {
				printf("\t\t\t0x%x %s\n", videntry->modeid, videntry->name);
			}
		}

		sRsrcVidMode *vidmode = NULL;
		TAILQ_FOREACH(vidmode, &iter->vidModes, next) {
			sRsrcVidNameEntry *videntry = NULL;
			printf("\tVideo Mode %d:\n", vidmode->modeid);
			
			TAILQ_FOREACH(videntry, &boardrsrc->vidNames, next) {
				if(vidmode->modeid == videntry->modeid) {
					printf("\t\tMode Name: %s\n", videntry->name);
					break;
				}
			}
			printf("\t\tDev Type: %d\n", vidmode->devtype);
			printf("\t\tPage Count: %d\n", vidmode->pages);
			printf("\t\tVideo Parameters:\n");
			printf("\t\t\tvpBaseOffset: %d\n", vidmode->vidParams->vpBaseOffset);
			printf("\t\t\tvpRowBytes: %d\n", vidmode->vidParams->vpRowBytes);
			printf("\t\t\tvpVersion: %d\n", vidmode->vidParams->vpVersion);
			printf("\t\t\tvpHRes: 0x%x\n", vidmode->vidParams->vpHRes);
			printf("\t\t\tvpVRes: 0x%x\n", vidmode->vidParams->vpVRes);
			printf("\t\t\tvpPixelType: %d\n", vidmode->vidParams->vpPixelType);
			printf("\t\t\tvpPixelSize: %d\n", vidmode->vidParams->vpPixelSize);
			printf("\t\t\tvpCmpCount: %d\n", vidmode->vidParams->vpCmpCount);
			printf("\t\t\tvpCmpSize: %d\n", vidmode->vidParams->vpCmpSize);
			printf("\t\t\tvpPlaneBytes: %d\n", vidmode->vidParams->vpPlaneBytes);

		}
	}

	exit(0);
}
