#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sysexits.h>
#include "declrom.h"

int main(int argc, char *argv[]) {
	sDeclRom *rom = sdeclrom_readfile(argv[1]);
	if(!rom) {
		exit(EX_SOFTWARE);
	}

	sResource *bootrom = calloc(1, sizeof(sResource));
	bootrom->type = calloc(1, sizeof(sRsrcType));
	bootrom->type->category = 2;
	bootrom->type->cType = 1;
	bootrom->type->drSW = 1;
	bootrom->type->drHW = 100;

	// The name of the resource *must* match the name of the driver
	bootrom->name = strdup("Romdrv");

	bootrom->drvrDir[0] = calloc(1, sizeof(sBlock));

	int drvrfd = open("romdrvr.768k", O_RDONLY);
	if(drvrfd < 0) {
		perror("Couldn't open romdrvr");
		exit(EX_SOFTWARE);
	}

	struct stat sb;
	if(fstat(drvrfd, &sb) != 0) {
		perror("Couldn't stat romdrvr");
		exit(EX_SOFTWARE);
	}

	bootrom->drvrDir[0]->len = sb.st_size + 4;
	bootrom->drvrDir[0]->data = malloc(sb.st_size);
	if(read(drvrfd, bootrom->drvrDir[0]->data, sb.st_size) != sb.st_size) {
		perror("Couldn't read romdrvr");
		exit(EX_SOFTWARE);
	}
	bootrom->drvrDir[0]->cpuid = 2;

	close(drvrfd);

	bootrom->bootRec = calloc(1, sizeof(sExecBlock));
	bootrom->bootRec->revision = 2;
	bootrom->bootRec->cpuid = 2;
	bootrom->bootRec->offset = 4;

	int bootfd = open("bootrec", O_RDONLY);
	if(bootfd < 0) {
		perror("Couldn't open bootrec");
		exit(EX_SOFTWARE);
	}
	if(fstat(bootfd, &sb) != 0) {
		perror("Couldn't stat bootrec");
		exit(EX_SOFTWARE);
	}
	bootrom->bootRec->len = sb.st_size + 12;
	bootrom->bootRec->code = malloc(sb.st_size);
	if(read(bootfd, bootrom->bootRec->code, sb.st_size) != sb.st_size) {
		perror("Couldn't read bootrec");
		exit(EX_SOFTWARE);
	}
	close(drvrfd);

	bootrom->flags = 2;
	bootrom->devid = 10;

	bootrom->minBaseOS = malloc(4);
	*bootrom->minBaseOS = 0;
	bootrom->minLen = malloc(4);
	*bootrom->minLen = 0xFFC0;
	
#if 0
	// Only relevant for board id, which we are not
	bootrom->vendorid = strdup("Rob Braun <bbraun@synack.net>");
	bootrom->revlevel = strdup("0.3");
	bootrom->date = strdup("July 2012");
#endif

	bootrom->rsrcid = 254;

	TAILQ_INSERT_BEFORE(TAILQ_LAST(&rom->resourcelist, rsrcq), bootrom, next);

	sdeclrom_writefile(rom, "mergedrom.bin");
	return 0;
}
